<?php
declare(strict_types=1);

namespace NL\NlMenubuilder\Domain\Model;


use TYPO3\CMS\Extbase\Domain\Model\FileReference;
use TYPO3\CMS\Extbase\DomainObject\AbstractEntity;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;

/***
 *
 * This file is part of the "Menu Builder" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2021
 *
 ***/
/**
 * MenuItem
 */
class MenuItem extends AbstractEntity
{
    const TYPE_PAGE = 0;
    const TYPE_LINK = 1;
    const TYPE_CONTENT = 2;
    const TYPE_SUBMENU = 3;

    /**
     * Type of the menu item
     *
     * @var int
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $type = 0;

    /**
     * Name of the menu item
     *
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $title = '';

    /**
     * subtitle
     *
     * @var string
     */
    protected $subtitle = '';

    /**
     * Optional link image
     *
     * @var FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $image = null;

    /**
     * link
     *
     * @var string
     */
    protected $link = '';

    /**
     * content
     *
     * @var ObjectStorage<TtContent>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $content = null;

    /**
     * items
     *
     * @var ObjectStorage<MenuItem>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $items = null;

    /**
     * page
     *
     * @var Page
     */
    protected $page = null;

    /**
     * __construct
     */
    public function __construct()
    {
        //Do not remove the next line: It would break the functionality
        $this->initStorageObjects();
    }

    /**
     * Initializes all ObjectStorage properties
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    protected function initStorageObjects()
    {
        $this->content = $this->content ?: new ObjectStorage();
        $this->items = $this->items ?: new ObjectStorage();
    }

    /**
     * Get type name of the menu item.
     *
     * @return string
     */
    public function getTypeName(): string
    {
        switch ($this->type) {
            case self::TYPE_PAGE:
                return 'Page';
            case self::TYPE_LINK:
                return 'Link';
            case self::TYPE_CONTENT:
                return 'Content';
            case self::TYPE_SUBMENU:
                return 'Submenu';
        }

        return 'Undefined';
    }

    /**
     * Returns the title
     *
     * @return string $title
     */
    public function getTitle(): string
    {
        return $this->title;
    }

    /**
     * Sets the title
     *
     * @param string $title
     * @return void
     */
    public function setTitle(string $title): void
    {
        $this->title = $title;
    }

    /**
     * Returns the type
     *
     * @return int $type
     */
    public function getType(): int
    {
        return $this->type;
    }

    /**
     * Sets the type
     *
     * @param int $type
     * @return void
     */
    public function setType(int $type): void
    {
        $this->type = $type;
    }

    /**
     * Returns the subtitle
     *
     * @return string $subtitle
     */
    public function getSubtitle(): string
    {
        return $this->subtitle;
    }

    /**
     * Sets the subtitle
     *
     * @param string $subtitle
     * @return void
     */
    public function setSubtitle(string $subtitle): void
    {
        $this->subtitle = $subtitle;
    }

    /**
     * Returns the image
     *
     * @return FileReference $image
     */
    public function getImage(): ?FileReference
    {
        return $this->image;
    }

    /**
     * Sets the image
     *
     * @param FileReference $image
     * @return void
     */
    public function setImage(FileReference $image): void
    {
        $this->image = $image;
    }

    /**
     * Returns the link
     *
     * @return string $link
     */
    public function getLink(): string
    {
        return $this->link;
    }

    /**
     * Sets the link
     *
     * @param string $link
     * @return void
     */
    public function setLink(string $link): void
    {
        $this->link = $link;
    }

    /**
     * Returns the content
     *
     * @return string $content
     */
    public function getContent(): string
    {
        $uids = array_map(function (TtContent $ce) {
            return $ce->getUid();
        }, $this->getContentRecords()->toArray());

        return implode(',', $uids);
    }

    /**
     * @return ObjectStorage
     */
    public function getContentRecords(): ObjectStorage
    {
        return $this->content;
    }

    /**
     * Adds a MenuItem
     *
     * @param MenuItem $item
     * @return void
     */
    public function addItem(MenuItem $item): void
    {
        $this->items->attach($item);
    }

    /**
     * Removes a MenuItem
     *
     * @param MenuItem $itemToRemove The MenuItem to be removed
     * @return void
     */
    public function removeItem(MenuItem $itemToRemove): void
    {
        $this->items->detach($itemToRemove);
    }

    /**
     * Returns the items
     *
     * @return ObjectStorage<MenuItem> $items
     */
    public function getItems(): ?ObjectStorage
    {
        return $this->items;
    }

    /**
     * Sets the items
     *
     * @param ObjectStorage<MenuItem> $items
     * @return void
     */
    public function setItems(ObjectStorage $items): void
    {
        $this->items = $items;
    }

    /**
     * Returns the page
     *
     * @return Page $page
     */
    public function getPage(): ?Page
    {
        return $this->page;
    }

    /**
     * Sets the page
     *
     * @param Page $page
     * @return void
     */
    public function setPage(Page $page): void
    {
        $this->page = $page;
    }
}
